"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var app;
var hmrRender;
var root;

function shouldReload() {
  return app && hmrRender && root;
}

var _default = {
  patch: function patch(inst) {
    if (!isDvaInstance(inst)) {
      throw Error('[hot.patch] app should be a `dva` instance');
    }

    var oldStart = inst.start; // eslint-disable-next-line

    inst.start = function (container) {
      if (isString(container)) {
        // eslint-disable-next-line
        container = document.querySelector(container);
      }

      if (isHTMLElement(container)) {
        app = inst;
        root = container;
        app.use({
          onHmr: function onHmr(render) {
            console.log('[dva-hot] enabled.');
            hmrRender = render;
          }
        });
      } else {
        app = undefined;
        root = undefined;
        hmrRender = undefined;
        console.warn('[dva-hot] disabled.');
      }

      return oldStart.call(inst, container);
    };

    return inst;
  },
  model: function model(sourceModule) {
    if (!isNodeModule(sourceModule)) {
      throw Error('[hot.model] should pass `module` first');
    }

    if (sourceModule.hot) {
      return function (model) {
        if (!isDvaModel(model)) {
          throw Error('[hot.model] model shoule be a obj and has `namespace` prop');
        }

        var namespace = model.namespace;
        sourceModule.hot.accept();
        sourceModule.hot.dispose(function () {
          if (namespace) {
            app.unmodel(namespace);
            namespace = undefined;
          }
        });

        if (shouldReload()) {
          try {
            app.model(model);
          } catch (e) {
            console.error('error', e);
          }
        }

        return model;
      };
    }

    return passthrough;
  },
  router: function router(sourceModule) {
    if (!isNodeModule(sourceModule)) {
      throw Error('[hot.router] should pass `module` first');
    }

    if (sourceModule.hot) {
      return function (router) {
        if (!isFunction(router)) {
          throw Error('[hot.router] router should be a function');
        }

        sourceModule.hot.accept();

        if (shouldReload()) {
          var renderException = function renderException(error) {
            var _require = require('react-dom'),
                render = _require.render;

            var _require2 = require('react'),
                createElement = _require2.createElement;

            var RedBox = require('redbox-react');

            render(createElement(RedBox || RedBox.default, {
              error: error
            }), root);
          };

          var newRender = function newRender(x) {
            try {
              hmrRender(x);
            } catch (error) {
              console.error('error', error);
              renderException(error);
            }
          };

          newRender(router);
        }

        return router;
      };
    }

    return passthrough;
  }
};
exports.default = _default;

function isHTMLElement(node) {
  return _typeof(node) === 'object' && node !== null && node.nodeType && node.nodeName;
}

function isString(str) {
  return typeof str === 'string';
}

function isFunction(func) {
  return typeof func === 'function';
}

function isDvaInstance(inst) {
  return inst && isFunction(inst.start) && isFunction(inst.model) && isFunction(inst.router) && isFunction(inst.use);
}

function isDvaModel(obj) {
  return obj && obj.namespace;
}

function isObject(obj) {
  return obj && _typeof(obj) === 'object';
}

function passthrough(x) {
  return x;
}

function isNodeModule(x) {
  return isObject(x) && 'id' in x;
}